@@ -25,6 +25,20 @@
 #include <qfile.h>
 #include <qurl.h>
 
+#include <QContactManager>
+using namespace QtMobility;
+#include <QContactPhoneNumber>
+#include <QContactName>
+
+#include <QContactEmailAddress>
+#include <QContactNote>
+#include <QContactBirthday>
+#include <QContactAddress>
+#include <QContactUrl>
+#include <QContactNickname>
+
+#include <QMaemo5InformationBox>
+
 //#define QT_NO_KEYWORDS
 //#include <libosso-abook/osso-abook-contact.h>
 //#include <libosso-abook/osso-abook-util.h>
@@ -44,6 +58,10 @@ QImage QrSink::getImage() {
     return image;
 }
 
+QString QrSink::getInformativeText() {
+    return barcodeInformativeText.left(60) + "...";
+}
+
 void QrSink::clickAction(QWidget *parentWindow) {
      //QDBusInterface *interface = new QDBusInterface("com.nokia.osso_browser","/com/nokia/osso_browser/request",
      //                                               "com.nokia.osso_browser");
@@ -53,6 +71,10 @@ void QrSink::clickAction(QWidget *parentWindow) {
         addVCard(barcodeText); //openGeneric(barcodeText); //
 
     }else if(type==QrSink::BizCard){
+        addMECARD(barcodeText);
+
+    }else if(type==QrSink::iModeMECARD){
+        addMECARD(barcodeText);
 
     }else if(type==QrSink::Install){
         openInstallFile(barcodeText);
@@ -130,98 +152,127 @@ void QrSink::barcodeAnalysed(QString barcodeType, QString barcodeData) {
             type=this->QrSink::vCard;
             buttonText.append("Open vCard");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData; // todo, parse and add name of person here
 
         }else if(barcodeData.indexOf("BIZCARD:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::BizCard;
             buttonText.append("Open BizCard");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData; // todo, parse and add name of person here
 
         }else if(barcodeData.indexOf("[install]",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::Install;
             buttonText.append("Install package");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData; // todo, parse and add name of app here
 
         }else if(barcodeData.indexOf("URL:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::URL;
             buttonText.append("Open URL");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData;
+
         }else if(barcodeData.indexOf("URLTO:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::URL;
             buttonText.append("Open URL");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData;
+
         }else if(barcodeData.indexOf("http://",0,Qt::CaseInsensitive)==0 || barcodeData.indexOf("https://",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::URL;
             buttonText.append("Open URL");
             barcodeText=barcodeData; // need to find where the http:// occurs in the string and trim it!
+            barcodeInformativeText = barcodeData;
 
         }else if(barcodeData.indexOf("CNTS:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::iMode;
             buttonText.append("Open iMode data");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData;
+
         }else if(barcodeData.indexOf("MECARD:",0,Qt::CaseInsensitive)==0){
-            type=this->QrSink::iMode;
-            buttonText.append("Open iMode data");
+            type=this->QrSink::iModeMECARD;
+            buttonText.append("Open iMode MECARD data");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData;
+
         }else if(barcodeData.indexOf("MATMSG:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::iMode;
             buttonText.append("Open iMode data");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData;
+
         }else if(barcodeData.indexOf("MEBKM:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::iMode;
             buttonText.append("Open iMode data");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData;
+
         }else if(barcodeData.indexOf("LAPL:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::iMode;
             buttonText.append("Open iMode data");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData;
 
         }else if(barcodeData.indexOf("mailto:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::Email;
             buttonText.append("Send e-mail");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData;
 
         }else if(barcodeData.indexOf("tel:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::PhoneNumber;
             buttonText.append("Use phone number");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData;
 
         }else if(barcodeData.indexOf("sms:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::SMS;
             buttonText.append("Open SMS");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData;
+
         }else if(barcodeData.indexOf("smsto:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::SMS;
             buttonText.append("Open SMS");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData;
 
         }else if(barcodeData.indexOf("mms:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::MMS;
             buttonText.append("Open MMS");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData;
+
         }else if(barcodeData.indexOf("mmsto:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::MMS;
             buttonText.append("Open MMS");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData;
 
         }else if(barcodeData.indexOf("geo:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::Geo;
             buttonText.append("Open geo location");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData;
 
         }else if(barcodeData.indexOf("youtube:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::YouTube;
             buttonText.append("Open YouTube link");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData;
 
         }else if(barcodeData.indexOf("ovi:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::Ovi;
             buttonText.append("Install from Ovi");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData;
 
         }else{
             type=this->QrSink::Unknown;
             buttonText.append("View text");
             barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData;
         }
 
 
@@ -598,4 +649,255 @@ void QrSink::addVCard(QString user_data)
 //    delete lib;
 }
 
+void QrSink::addMECARD(QString user_data)
+{
+
+    // remove the MECARD: bit at the start
+    if(user_data.indexOf("MECARD:",0,Qt::CaseInsensitive)==0){
+        user_data = user_data.mid(7);
+    } else if(user_data.indexOf("BIZCARD:",0,Qt::CaseInsensitive)==0){
+        user_data = user_data.mid(8);
+    }
+
+    // find a contact manager
+    QStringList availableManagers = QContactManager::availableManagers();
+    QContactManager *manager;
+
+    qDebug() << "Entered QrSink::addMECARD()";
+
+    for(int managerIdx = 0; managerIdx < availableManagers.count(); managerIdx++) {
+        manager = new QContactManager(availableManagers.at(managerIdx));
+
+        if(manager) {
+            //QList<QContactLocalId> contacts = manager->contacts();
+            break;
+        }
+    }
+
+    if(!manager) {
+        qDebug() << "Failed to open a contact manager";
+        return;
+    }
+
+
+    // create a new contact
+    QContact newContact;
+
+    // parse the string
+    // ; record separators
+    // : field name sep
+    // ;; ends string
+
+    QStringList records = user_data.split(";", QString::SkipEmptyParts);
+    for(int recordIdx = 0; recordIdx < records.length(); recordIdx++) {
+        QString currentRecord = records.at(recordIdx);
+
+        // see what this record encodes
+        if(currentRecord.indexOf("N:",0,Qt::CaseSensitive)==0){
+            // name
+            QString nameString(currentRecord.mid(2)); // chop off the N:
+
+            QContactName newContactName;
+
+            if(nameString.contains(",")){
+                // comma is used to separate first and last names
+                QStringList nameItems = nameString.split(",");
+                QString lastName = nameItems.at(0);
+                QString firstName = nameItems.at(1);
+                newContactName.setFirstName(firstName);
+                newContactName.setLastName(lastName);
+                qDebug() << "QrSink::addMECARD() found first/last names: " << firstName << " " << lastName;
+            }else{
+                // just a name, no first or last
+                // so what should we do?
+                newContactName.setCustomLabel(nameString);
+                qDebug() << "QrSink::addMECARD() found name: " << nameString;
+            }
+            if(newContact.saveDetail(&newContactName))
+                qDebug() << "Saved name";
+            else
+                qDebug() << "Failed to save name";
+
+        }else if(currentRecord.indexOf("SOUND:",0,Qt::CaseSensitive)==0){
+            // no idea what this does
+        }else if(currentRecord.indexOf("TEL:",0,Qt::CaseSensitive)==0){
+            // telephone number
+
+            QString phonenumberString(currentRecord.mid(4)); // chop off the TEL:
+            QContactPhoneNumber newContactNumber;
+
+            newContactNumber.setNumber(phonenumberString);
+            // should be have a subtype here?
+
+            qDebug() << "QrSink::addMECARD() found phone number: " << phonenumberString;
+            if(newContact.saveDetail(&newContactNumber))
+                qDebug() << "Saved phone number";
+            else
+                qDebug() << "Failed to save phone number";
+
+
+        }else if(currentRecord.indexOf("TEL-AV:",0,Qt::CaseSensitive)==0){
+            // videophone number
+            QString phonenumberString(currentRecord.mid(7)); // chop off the TEL-AV:
+            QContactPhoneNumber newContactNumber;
+
+            newContactNumber.setNumber(phonenumberString);
+            newContactNumber.setSubTypes(QContactPhoneNumber::SubTypeVideo);
+
+            qDebug() << "QrSink::addMECARD() found video phone number: " << phonenumberString;
+            if(newContact.saveDetail(&newContactNumber))
+                qDebug() << "Saved video phone number";
+            else
+                qDebug() << "Failed to save video phone number";
+
+
+        }else if(currentRecord.indexOf("EMAIL:",0,Qt::CaseSensitive)==0){
+            // email address
+            QString emailString(currentRecord.mid(6)); // chop off the EMAIL:
+            QContactEmailAddress newEmail;
+
+            newEmail.setEmailAddress(emailString);
+
+            qDebug() << "QrSink::addMECARD() found email address: " << emailString;
+            if(newContact.saveDetail(&newEmail))
+                qDebug() << "Saved email";
+            else
+                qDebug() << "Failed to save email";
+
+
+
+        }else if(currentRecord.indexOf("NOTE:",0,Qt::CaseSensitive)==0){
+            // memo
+            QString noteString(currentRecord.mid(5)); // chop off the NOTE:
+            QContactNote newNote;
+
+            newNote.setNote(noteString);
+            qDebug() << "QrSink::addMECARD() found note: " << noteString;
+
+            if(newContact.saveDetail(&newNote))
+                qDebug() << "Saved note";
+            else
+                qDebug() << "Failed to save note";
+
+
+        }else if(currentRecord.indexOf("BDAY:",0,Qt::CaseSensitive)==0){
+            // birthday
+            QString dateString(currentRecord.mid(5)); // chop off the BDAY:
+            QString year = dateString.mid(0,4);
+            QString month = dateString.mid(4,2);
+            QString day = dateString.mid(6,2);
+            QDate bdayDate(year.toInt(), month.toInt(), day.toInt());
+            QContactBirthday newBDay;
+
+            newBDay.setDate(bdayDate);
+            qDebug() << "QrSink::addMECARD() found birthday: " << bdayDate;
+
+            if(newContact.saveDetail(&newBDay))
+                qDebug() << "Saved birthday";
+            else
+                qDebug() << "Failed to save birthday";
+
+
+
+        }else if(currentRecord.indexOf("ADR:",0,Qt::CaseSensitive)==0){
+            // address
+            QString adrString(currentRecord.mid(4)); // chop off the ADR:
+            QStringList adrItems = adrString.split(",");
+            QContactAddress newAdr;
+
+            newAdr.setPostOfficeBox(adrItems.at(0)); // PO box
+            newAdr.setStreet(adrItems.at(1) + QString(" ") + adrItems.at(2)); // room number + house number
+            newAdr.setLocality(adrItems.at(3)); // city
+            newAdr.setRegion(adrItems.at(4)); // prefecture
+            newAdr.setPostcode(adrItems.at(5)); // zip code
+            newAdr.setCountry(adrItems.at(6)); // country
+
+            qDebug() << "QrSink::addMECARD() found address (PO box): " << adrItems.at(0);
+            qDebug() << "QrSink::addMECARD() found address (room number): " << adrItems.at(1);
+            qDebug() << "QrSink::addMECARD() found address (house number): " << adrItems.at(2);
+            qDebug() << "QrSink::addMECARD() found address (city): " << adrItems.at(3);
+            qDebug() << "QrSink::addMECARD() found address (prefecture): " << adrItems.at(4);
+            qDebug() << "QrSink::addMECARD() found address (zip code): " << adrItems.at(5);
+            qDebug() << "QrSink::addMECARD() found address (country): " << adrItems.at(6);
+
+            if(newContact.saveDetail(&newAdr))
+                qDebug() << "Saved address";
+            else
+                qDebug() << "Failed to save address";
+
+
+
+        }else if(currentRecord.indexOf("URL:",0,Qt::CaseSensitive)==0){
+            // webpage
+            QString urlString(currentRecord.mid(4)); // chop off the URL:
+            QContactUrl newURL;
+
+            newURL.setUrl(urlString);
+            qDebug() << "QrSink::addMECARD() found url: " << urlString;
+
+            if(newContact.saveDetail(&newURL))
+                qDebug() << "Saved url";
+            else
+                qDebug() << "Failed to save url";
+
+
+        }else if(currentRecord.indexOf("NICKNAME:",0,Qt::CaseSensitive)==0){
+            // nickname
+            QString nickString(currentRecord.mid(9)); // chop off the NICKNAME:
+            QContactNickname newNick;
+
+            newNick.setNickname(nickString);
+            qDebug() << "QrSink::addMECARD() found nickname: " << nickString;
+
+            if(newContact.saveDetail(&newNick))
+                qDebug() << "Saved nickname";
+            else
+                qDebug() << "Failed to save nickname";
+
+
+        }else{
+            // unrecognised!
+        }
+    }
+
+
+    // we should now see if the contact already exists and if so give the option to replace/merge/add
+
+
+    // should we add a signal/slot pair so that plugins can signal their success/lack thereof to the ui and the user?
+
+    /* Save the contact */
+    if(manager->saveContact(&newContact)){
+        //QMaemo5InformationBox::information (this->parent.parent, QString("Added MECARD"), 1000);
+        qDebug() << "Added contact successfully";
+    }else{
+        //QMaemo5InformationBox::information (this->parent.parent, QString("Failed to add MECARD :("), 1000);
+        qDebug() << "Could not add contact, error:" << manager->error();
+
+    }
+
+
+//    // see if this contact already exists
+//
+//    QContactDetailFilter nameFilter;
+//    nameFilter.setDetailDefinitionName(QContactName::DefinitionName, QContactName::FieldNumber);
+//    nameFilter.setValue(incomingCallNbr);
+//    nameFilter.setMatchFlags(QContactFilter::MatchExactly);
+//
+//    QList<QContactLocalId> matchingContacts = cm->contacts(nameFilter);
+//    if (matchingContacts.size() == 0) {
+//        qDebug() << "Incoming call from unknown contact (" << incomingCallNbr << ")";
+//    } else {
+//        QContact match = cm->contact(matchingContacts.at(0));
+//        qDebug() << "Incoming call from"
+//                 << match.displayLabel()
+//                 << "(" << incomingCallNbr << ")";
+//    }
+//
+//
+
+
+}
+
+
 
