@@ -20,6 +20,7 @@ public:
         vCard,
         BizCard,
         iMode,
+        iModeMECARD,
         Geo,
         YouTube,
         Install,
@@ -30,6 +31,7 @@ public:
     QString getName() {return "QR-code sink plugin";}
     QString getText();
     QImage getImage();
+    QString getInformativeText();
     bool isReady();
     void clickAction(QWidget *parentWindow);
     // other
@@ -41,10 +43,12 @@ private:
     void addVCard(QString user_data);
     void openInstallFile(QString data);
     void openBrowser(QString uri);
+    void addMECARD(QString user_data);
 
     bool ready;
     QString buttonText;
     QString barcodeText;
+    QString barcodeInformativeText;
     int type;
 
     QImage image;
